/**
* @file Callback.c
* @author Hugo Tavares nº 27764 (hugo.tavares@ua.pt)
* @brief Ficheiro que contém as Callbacks geradas pelos widgets
* 
* Este ficheiro contém todas as Callbacks que são chamadas sempre
* que ocorra um determinado evento num determinado Widget.
*/


#define _CALL_BACK_
#include "SALbot_Interface.h"

/**
* @fn on_SAL1_bt_ligar_clicked(GtkWidget *widget, gpointer user_data)
* CallBack que trata a resposta ao clique no botão ligar. \n
* Pede ao Pai para enviar uma mensagem ao dispositivo para começar a enviar dados.
* @param widget Corresponde ao botão.
* @param user_data Dados passados pelo widget
*/
void on_SAL1_bt_ligar_clicked(GtkWidget *widget, gpointer user_data)
{
       int pid = getppid();
       
       while(!pf_msgG->request_to_send){}
       pf_msgG->send_msg.SALbot_dest = SALbot1;
       pf_msgG->send_msg.Tipo = start_stop_cmd;
       
       if(EstadoG.SAL1.Modo_op == Idle)
       {
	      pf_msgG->send_msg.Cmd = start_cmd;
       }
       else
       {
	      pf_msgG->send_msg.Cmd = stop_cmd;
       }
       
       pf_msgG->request_to_send = FALSE;
       kill(pid,SIGUSR1);
}

/**
* @fn on_SAL2_bt_ligar_clicked(GtkWidget *widget, gpointer user_data)
* CallBack que trata a resposta ao clique no botão ligar. \n
* Pede ao Pai para enviar uma mensagem ao dispositivo para começar a enviar dados.
* @param widget Corresponde ao botão.
* @param user_data Dados passados pelo widget
*/
void on_SAL2_bt_ligar_clicked(GtkWidget *widget, gpointer user_data)
{
       int pid = getppid();
       
       while(!pf_msgG->request_to_send){}
       pf_msgG->send_msg.SALbot_dest = SALbot2;
       pf_msgG->send_msg.Tipo = start_stop_cmd;
       
       if(EstadoG.SAL2.Modo_op == Idle)
       {
	      pf_msgG->send_msg.Cmd = start_cmd;
       }
       else
       {
	      pf_msgG->send_msg.Cmd = stop_cmd;
       }
       
       pf_msgG->request_to_send = FALSE;
       kill(pid,SIGUSR1);
}

/**
* @fn on_SAL1_bt_modo_op_clicked(GtkWidget *widget, gpointer user_data)
* CallBack que trata a resposta ao clique no botão modo de operação. \n
* Pede ao Pai para enviar uma mensagem ao dispositivo para lhe modar o modo de operação.
* @param widget Corresponde ao botão.
* @param user_data Dados passados pelo widget
*/
void on_SAL1_bt_modo_op_clicked(GtkWidget *widget, gpointer user_data)
{
       int pid = getppid();
       
       while(!pf_msgG->request_to_send){}
       pf_msgG->send_msg.SALbot_dest = SALbot1;
       pf_msgG->send_msg.Tipo = op_mode_cmd;
       
       if(EstadoG.SAL1.Modo_op == Manual)
       {
	      pf_msgG->send_msg.Cmd = autonomo;
	      EstadoG.SAL1.Modo_op = Autonomo;
       }
       else
       {
	      pf_msgG->send_msg.Cmd = manual;
	      EstadoG.SAL1.Modo_op = Manual;
       }
       
       pf_msgG->request_to_send = FALSE;
       kill(pid,SIGUSR1);
}

/**
* @fn on_SAL2_bt_modo_op_clicked(GtkWidget *widget, gpointer user_data)
* CallBack que trata a resposta ao clique no botão modo de operação. \n
* Pede ao Pai para enviar uma mensagem ao dispositivo para lhe modar o modo de operação.
* @param widget Corresponde ao botão.
* @param user_data Dados passados pelo widget
*/
void on_SAL2_bt_modo_op_clicked(GtkWidget *widget, gpointer user_data)
{
       int pid = getppid();
       
       while(!pf_msgG->request_to_send){}
       pf_msgG->send_msg.SALbot_dest = SALbot2;
       pf_msgG->send_msg.Tipo = op_mode_cmd;
       
       if(EstadoG.SAL2.Modo_op == Manual)
       {
	      pf_msgG->send_msg.Cmd = autonomo;
	      EstadoG.SAL2.Modo_op = Autonomo;
       }
       else
       {
	      pf_msgG->send_msg.Cmd = manual;
	      EstadoG.SAL2.Modo_op = Manual;
       }
       
       pf_msgG->request_to_send = FALSE;
       kill(pid,SIGUSR1);
}

/**
* @fn on_SAL1_bt_req_sens_dist_clicked(GtkWidget *widget, gpointer user_data)
* CallBack que trata a resposta ao clique no botão de requisição de dados dos \b Sensores \b de \b distância. \n
* Pede ao Pai para enviar uma mensagem ao dispositivo para lhe ser enviado dados dos sensores de distância.
* @param widget Corresponde ao botão.
* @param user_data Dados passados pelo widget
*/
void on_SAL1_bt_req_sens_dist_clicked(GtkWidget *widget, gpointer user_data)
{
       if(EstadoG.SAL1.Modo_op == Manual)
       {
	      int pid = getppid();
	      
	      while(!pf_msgG->request_to_send){}
	      pf_msgG->send_msg.SALbot_dest = SALbot1;
	      pf_msgG->send_msg.Tipo = req_cmd;
	      pf_msgG->send_msg.Cmd = sens_dist;
	      
	      pf_msgG->request_to_send = FALSE;
	      kill(pid,SIGUSR1);
       }
}

/**
* @fn on_SAL2_bt_req_sens_dist_clicked(GtkWidget *widget, gpointer user_data)
* CallBack que trata a resposta ao clique no botão de requisição de dados dos \b Sensores \b de \b distância. \n
* Pede ao Pai para enviar uma mensagem ao dispositivo para lhe ser enviado dados dos sensores de distância.
* @param widget Corresponde ao botão.
* @param user_data Dados passados pelo widget
*/
void on_SAL2_bt_req_sens_dist_clicked(GtkWidget *widget, gpointer user_data)
{
       if(EstadoG.SAL2.Modo_op == Manual)
       {
	      int pid = getppid();
	      
	      while(!pf_msgG->request_to_send){}
	      pf_msgG->send_msg.SALbot_dest = SALbot2;
	      pf_msgG->send_msg.Tipo = req_cmd;
	      pf_msgG->send_msg.Cmd = sens_dist;
	      
	      pf_msgG->request_to_send = FALSE;
	      kill(pid,SIGUSR1);
       }
}

/**
* @fn on_SAL1_bt_req_sens_bumper_clicked(GtkWidget *widget, gpointer user_data)
* CallBack que trata a resposta ao clique no botão de requisição de dados dos \b Sensores \b de \b distância. \n
* Pede ao Pai para enviar uma mensagem ao dispositivo para lhe ser enviado dados dos sensores de distância.
* @param widget Corresponde ao botão.
* @param user_data Dados passados pelo widget
*/
void on_SAL1_bt_req_sens_bumper_clicked(GtkWidget *widget, gpointer user_data)
{
       if(EstadoG.SAL1.Modo_op == Manual)
       {
	      int pid = getppid();
	      
	      while(!pf_msgG->request_to_send){}
	      pf_msgG->send_msg.SALbot_dest = SALbot1;
	      pf_msgG->send_msg.Tipo = req_cmd;
	      pf_msgG->send_msg.Cmd = sens_bumper;
	      
	      pf_msgG->request_to_send = FALSE;
	      kill(pid,SIGUSR1);
       }
}

/**
* @fn on_SAL2_bt_req_sens_bumper_clicked(GtkWidget *widget, gpointer user_data)
* CallBack que trata a resposta ao clique no botão de requisição de dados dos \b Sensores \b de \b distância. \n
* Pede ao Pai para enviar uma mensagem ao dispositivo para lhe ser enviado dados dos sensores de distância.
* @param widget Corresponde ao botão.
* @param user_data Dados passados pelo widget
*/
void on_SAL2_bt_req_sens_bumper_clicked(GtkWidget *widget, gpointer user_data)
{
       if(EstadoG.SAL2.Modo_op == Manual)
       {
	      int pid = getppid();
	      
	      while(!pf_msgG->request_to_send){}
	      pf_msgG->send_msg.SALbot_dest = SALbot2;
	      pf_msgG->send_msg.Tipo = req_cmd;
	      pf_msgG->send_msg.Cmd = sens_bumper;
	      
	      pf_msgG->request_to_send = FALSE;
	      kill(pid,SIGUSR1);
       }
}

/**
* @fn on_notebook1_select_page(GtkWidget *widget, gpointer user_data)
 */
void on_notebook1_select_page(GtkWidget *widget, gpointer user_data)
{
       int page_n, page_s;
       GtkWidget *notebook;
       
       page_n = gtk_notebook_get_current_page ((GtkNotebook *) widget);
       switch(page_n)
       {
	      case 1:
		     notebook = glade_xml_get_widget(xmlG,"notebook2");
		     page_s = gtk_notebook_get_current_page ((GtkNotebook *) notebook);
		     if(page_s == 1)
		     {
			    //gtk_widget_grab_default(DataBoxG.SAL1_box);
			    gtk_widget_grab_focus(DataBoxG.SAL1_box);
		     }
		     break;
	      case 0:
		     notebook = glade_xml_get_widget(xmlG,"notebook3");
		     page_s = gtk_notebook_get_current_page ((GtkNotebook *) notebook);
		     if(page_s == 1)
		     {
			    //gtk_widget_grab_default(DataBoxG.SAL2_box);
			    gtk_widget_grab_focus(DataBoxG.SAL2_box);
		     }
		     break;
       }
}

/**
* @fn on_notebook2_select_page(GtkWidget *widget, gpointer user_data)
*/
void on_notebook2_select_page(GtkWidget *widget, gpointer user_data)
{
       int page_n;
       
       page_n = gtk_notebook_get_current_page ((GtkNotebook *) widget);
       switch(page_n)
       {
	      case 1:
		     break;
	      case 0:
		     //gtk_widget_grab_default(DataBoxG.SAL1_box);
		     gtk_widget_grab_focus(DataBoxG.SAL1_box);
		     break;
       }
}

/**
* @fn on_notebook3_select_page(GtkWidget *widget, gpointer user_data)
*/
void on_notebook3_select_page(GtkWidget *widget, gpointer user_data)
{
       int page_n;
       
       page_n = gtk_notebook_get_current_page ((GtkNotebook *) widget);
       switch(page_n)
       {
	      case 1:
		     break;
	      case 0:
		     //gtk_widget_grab_default(DataBoxG.SAL2_box);
		     gtk_widget_grab_focus(DataBoxG.SAL2_box);
		     break;
       }
}

/**
* @fn SAL1_key_pressed(GtkDatabox *box, GdkEventkey *event)
 */
void SAL1_key_pressed(GtkDatabox *box, GdkEventKey *event, GtkDataboxMarkers * markers)
{
       if(event->type != GDK_KEY_PRESS)
       {
	      return;
       }
       else
       {
	      switch(event->keyval)
	      {
		     case GDK_h:
		     case GDK_H:
			    gtk_databox_set_visible_limits(GTK_DATABOX(box),DataBoxG.SAL1_view.minv_x,DataBoxG.SAL1_view.maxv_x,DataBoxG.SAL1_view.maxv_y,DataBoxG.SAL1_view.minv_y);
			    break;
		     default:
			    break;
	      }
       }
}

/**
* @fn SAL2_key_pressed(GtkDatabox *box, GdkEventkey *event)
*/
void SAL2_key_pressed(GtkDatabox *box, GdkEventKey *event, GtkDataboxMarkers * markers)
{
       if(event->type != GDK_KEY_PRESS)
       {
	      return;
       }
       else
       {
	      switch(event->keyval)
	      {
		     case GDK_h:
		     case GDK_H:
			    gtk_databox_set_visible_limits(GTK_DATABOX(box),DataBoxG.SAL2_view.minv_x,DataBoxG.SAL2_view.maxv_x,DataBoxG.SAL2_view.maxv_y,DataBoxG.SAL2_view.minv_y);
			    break;
		     default:
			    break;
	      }
       }
}

/**
* @fn on_SAL1_bt_req_Q_table_clicked(GtkWidget *widget, gpointer user_data)
* CallBack que trata a resposta ao clique no botão de requisição de dados dos \b Sensores \b de \b distância. \n
* Pede ao Pai para enviar uma mensagem ao dispositivo para lhe ser enviado dados dos sensores de distância.
* @param widget Corresponde ao botão.
* @param user_data Dados passados pelo widget
*/
void on_SAL1_bt_req_Q_table_clicked(GtkWidget *widget, gpointer user_data)
{
       if(EstadoG.SAL1.Modo_op != Learning && EstadoG.SAL1.Modo_op != Autonomo)
       {
	      int pid = getppid();
	      
	      while(!pf_msgG->request_to_send){}
	      pf_msgG->send_msg.SALbot_dest = SALbot1;
	      pf_msgG->send_msg.Tipo = req_cmd;
	      pf_msgG->send_msg.Cmd = learn_read;
	      
	      pf_msgG->request_to_send = FALSE;
	      kill(pid,SIGUSR1);
       }
}

/**
* @fn on_SAL2_bt_req_Q_table_clicked(GtkWidget *widget, gpointer user_data)
* CallBack que trata a resposta ao clique no botão de requisição de dados dos \b Sensores \b de \b distância. \n
* Pede ao Pai para enviar uma mensagem ao dispositivo para lhe ser enviado dados dos sensores de distância.
* @param widget Corresponde ao botão.
* @param user_data Dados passados pelo widget
*/
void on_SAL2_bt_req_Q_table_clicked(GtkWidget *widget, gpointer user_data)
{
       if(EstadoG.SAL2.Modo_op != Learning && EstadoG.SAL2.Modo_op != Autonomo)
       {
	      int pid = getppid();
	      
	      
	      while(!pf_msgG->request_to_send){}
	      pf_msgG->send_msg.SALbot_dest = SALbot2;
	      pf_msgG->send_msg.Tipo = req_cmd;
	      pf_msgG->send_msg.Cmd = learn_read;
       
	      pf_msgG->request_to_send = FALSE;
	      kill(pid,SIGUSR1);
       }
}

/**
* @fn on_menu_limpar_activate(GtkWidget *widget, gpointer user_data)
* CallBack que trata a resposta ao clique no botão de menu limpar.\n
* Limpa todos os campos da interface gráfica, bem como todas as
* variáveis de estdado.
* @param widget Corresponde ao botão.
* @param user_data Dados passados pelo widget
*/
void on_menu_limpar_activate(GtkWidget *widget, gpointer user_data)
{
       
}

/**
* @fn on_menu_guardar_activate(GtkWidget *widget,gpointer user_data)
* CallBack que trata a resposta ao clique no botão de menu guardar.\n
* Guarda os dados em dois ficheiros, um de texto e uma imagem.
* @param widget Corresponde ao botão.
* @param user_data Dados passados pelo widget
*/
void on_menu_guardar_activate(GtkWidget *widget,gpointer user_data)
{
       int page_n;
       GtkWidget *notebook= glade_xml_get_widget(xmlG,"notebook1");
       
       page_n = gtk_notebook_get_current_page ((GtkNotebook *) notebook);
       switch(page_n)
       {
	      case 0:
		     save_dialog(1);
		     save_pgr_dialog(1);
		     break;
	      case 1:
		     save_dialog(2);
		     save_pgr_dialog(2);
		     break;
       }
}

/**
* @fn on_menu_abrir_activate(GtkWidget *widget,gpointer user_data)
* CallBack que trata a resposta ao clique no botão de menu abrir.\n
* Carrega os dados guardados.
* @param widget Corresponde ao botão.
* @param user_data Dados passados pelo widget
*/
void on_menu_abrir_activate(GtkWidget *widget, gpointer user_data)
{
       int page_n;
	   GtkWidget *notebook= glade_xml_get_widget(xmlG,"notebook1");
	   
	   page_n = gtk_notebook_get_current_page ((GtkNotebook *) notebook);
	   switch(page_n)
	   {
		  case 0:
			 load_dialog(1);
			 load_pgr_dialog(1);
			 Actualiza_graph_SAL1();
			 break;
		  case 1:
			 load_dialog(2);
			 load_pgr_dialog(2);
			 Actualiza_graph_SAL2();
			 break;
	   }
}

/**
* @fn void on_SAL1_bt_learning_toggled(GtkWidget *widget, gpointer user_data)
* 
*/
void on_SAL1_bt_learning_toggled(GtkWidget *widget, gpointer user_data)
{
       int pid = getppid();
       
       while(!pf_msgG->request_to_send){}
       pf_msgG->send_msg.SALbot_dest = SALbot1;
       pf_msgG->send_msg.Tipo = op_mode_cmd;
       pf_msgG->send_msg.Cmd = learning;
       
       pf_msgG->request_to_send = FALSE;
       kill(pid,SIGUSR1);
}

/**
* @fn void on_SAL2_bt_learning_toggled(GtkWidget *widget, gpointer user_data)
* 
*/
void on_SAL2_bt_learning_toggled(GtkWidget *widget, gpointer user_data)
{
       int pid = getppid();
       
       while(!pf_msgG->request_to_send){}
       pf_msgG->send_msg.SALbot_dest = SALbot2;
       pf_msgG->send_msg.Tipo = op_mode_cmd;
       pf_msgG->send_msg.Cmd = learning;
       
       pf_msgG->request_to_send = FALSE;
       kill(pid,SIGUSR1);
}

/**
* @fn void on_SAL1_bt_send_Q_table_clicked(GtkWidget *widget, gpointer user_data)
* 
*/
void on_SAL1_bt_send_Q_table_clicked(GtkWidget *widget, gpointer user_data)
{
       int a,s,pid = getppid();
       unsigned int Addr;
       signed char val;
       
       GtkWidget *parent_win = gtk_window_new(GTK_WINDOW_POPUP);
       GtkWidget *dialog_warn = gtk_message_dialog_new((GtkWindow *)parent_win,
													GTK_DIALOG_MODAL,
													GTK_MESSAGE_WARNING,
													GTK_BUTTONS_YES_NO,
													"Deseja realmente apagar a tabela de aprendizagem do robô e enviar a nova tabela de aprendizagem?");
       
       if(gtk_dialog_run(GTK_DIALOG(dialog_warn))==GTK_RESPONSE_YES)
       {	      
	      while(!pf_msgG->request_to_send){}
	      pf_msgG->send_msg.SALbot_dest = SALbot1;
	      pf_msgG->send_msg.Tipo = req_cmd;
	      pf_msgG->send_msg.Cmd = Q_reset;
	      
	      pf_msgG->request_to_send = FALSE;
	      kill(pid,SIGUSR1);
	      
	      while(!pf_msgG->request_to_send){}
	      pf_msgG->send_msg.SALbot_dest = SALbot1;
	      pf_msgG->send_msg.Tipo = send_pgr;
	      pf_msgG->send_msg.Pgr_send.It = EstadoG.SAL1.Learn_iteration;
	      pf_msgG->send_msg.Pgr_send.Col = EstadoG.SAL1.Learn_progress[EstadoG.SAL1.Learn_iteration];
	      pf_msgG->send_msg.Pgr_send.Eps = (unsigned int)(EstadoG.SAL1.Learn_epsilon[EstadoG.SAL1.Learn_iteration] * 1000);
	      
	      pf_msgG->request_to_send = FALSE;
	      kill(pid,SIGUSR1);
	      
	      for(s=0;s<Num_states_;s++)
	      {
		     for(a=0;a<Num_actions_;a++)
		     {
			    val = EstadoG.SAL1.Q_table[a][s];
			    
			    if(val != 0)
			    {
				   while(!pf_msgG->request_to_send){}
				   Addr = (s*Num_actions_) + a;
				   pf_msgG->send_msg.SALbot_dest = SALbot1;
				   pf_msgG->send_msg.Tipo = send_Q;
				   pf_msgG->send_msg.Q_send.Addr_H = (unsigned char)(Addr>>8)&0xFF;
				   pf_msgG->send_msg.Q_send.Addr_L = (unsigned char)Addr&0xFF;
				   pf_msgG->send_msg.Q_send.val = (unsigned char)val;

				   pf_msgG->request_to_send = FALSE;
				   kill(pid,SIGUSR1);
			    }
		     }
	      }
       }
       gtk_widget_destroy(dialog_warn);
}

/**
* @fn void on_SAL2_bt_send_Q_table_clicked(GtkWidget *widget, gpointer user_data)
* 
*/
void on_SAL2_bt_send_Q_table_clicked(GtkWidget *widget, gpointer user_data)
{
       int a,s,pid = getppid();
       unsigned int Addr;
       signed char val;
       
       GtkWidget *parent_win = gtk_window_new(GTK_WINDOW_POPUP);
       GtkWidget *dialog_warn = gtk_message_dialog_new((GtkWindow *)parent_win,
							GTK_DIALOG_MODAL,
							GTK_MESSAGE_WARNING,
							GTK_BUTTONS_YES_NO,
							"Deseja realmente apagar a tabela de aprendizagem do robô e enviar a nova tabela de aprendizagem?");
							
       if(gtk_dialog_run(GTK_DIALOG(dialog_warn))==GTK_RESPONSE_YES)
       {	      
	      while(!pf_msgG->request_to_send){}
	      pf_msgG->send_msg.SALbot_dest = SALbot2;
	      pf_msgG->send_msg.Tipo = req_cmd;
	      pf_msgG->send_msg.Cmd = Q_reset;
	      
	      pf_msgG->request_to_send = FALSE;
	      kill(pid,SIGUSR1);
	      
	      while(!pf_msgG->request_to_send){}
	      pf_msgG->send_msg.SALbot_dest = SALbot2;
	      pf_msgG->send_msg.Tipo = send_pgr;
	      pf_msgG->send_msg.Pgr_send.It = EstadoG.SAL2.Learn_iteration;
	      pf_msgG->send_msg.Pgr_send.Col = EstadoG.SAL2.Learn_progress[EstadoG.SAL2.Learn_iteration];
	      pf_msgG->send_msg.Pgr_send.Eps = (unsigned int)(EstadoG.SAL2.Learn_epsilon[EstadoG.SAL2.Learn_iteration] * 1000);
	      
	      pf_msgG->request_to_send = FALSE;
	      kill(pid,SIGUSR1);
	      
	      for(s=0;s<Num_states_;s++)
	      {
		     for(a=0;a<Num_actions_;a++)
		     {
			    val = EstadoG.SAL2.Q_table[a][s];
			    
			    if(val != 0)
			    {
				   while(!pf_msgG->request_to_send){}
				   Addr = (s*Num_actions_) + a;
				   pf_msgG->send_msg.SALbot_dest = SALbot2;
				   pf_msgG->send_msg.Tipo = send_Q;
				   pf_msgG->send_msg.Q_send.Addr_H = (unsigned char)(Addr>>8)&0xFF;
				   pf_msgG->send_msg.Q_send.Addr_L = (unsigned char)Addr&0xFF;
				   pf_msgG->send_msg.Q_send.val = (unsigned char)val;
				   
				   pf_msgG->request_to_send = FALSE;
				   kill(pid,SIGUSR1);
			    }
		     }
	      }
       }
       gtk_widget_destroy(dialog_warn);
}

/**
* @fn void on_SAL1_bt_load_Q_clicked(GtkWidget *widget, gpointer user_data)
* 
*/
void on_SAL1_bt_load_Q_clicked(GtkWidget *widget, gpointer user_data)
{   
       int i,j;
       GtkWidget *parent_win = gtk_window_new(GTK_WINDOW_POPUP);
       GtkWidget *dialog_warn = gtk_message_dialog_new((GtkWindow *)parent_win,
													GTK_DIALOG_MODAL,
													GTK_MESSAGE_WARNING,
													GTK_BUTTONS_YES_NO,
													"Deseja realmente carregar a tabela de aprendizagem?\nA tabela actual para o SALbot 1 irá ser substituida.");
       
       if(gtk_dialog_run(GTK_DIALOG(dialog_warn))==GTK_RESPONSE_YES)
       {
	      for(i=0;i<Num_actions_;i++)
	      {
		     for(j=0;j<Num_states_;j++)
		     {
			    EstadoG.SAL1.Q_table[i][j] = 0;
		     }
	      }
	      load_dialog(1);
       }
       gtk_widget_destroy(dialog_warn);
}

/**
* @fn void on_SAL2_bt_load_Q_clicked(GtkWidget *widget, gpointer user_data)
* 
*/
void on_SAL2_bt_load_Q_clicked(GtkWidget *widget, gpointer user_data)
{   
       int i,j;
       GtkWidget *parent_win = gtk_window_new(GTK_WINDOW_POPUP);
       GtkWidget *dialog_warn = gtk_message_dialog_new((GtkWindow *)parent_win,
													GTK_DIALOG_MODAL,
													GTK_MESSAGE_WARNING,
													GTK_BUTTONS_YES_NO,
													"Deseja realmente carregar a tabela de aprendizagem?\nA tabela actual para o SALbot 2 irá ser substituida.");
       
       if(gtk_dialog_run(GTK_DIALOG(dialog_warn))==GTK_RESPONSE_YES)
       {
	      for(i=0;i<Num_actions_;i++)
	      {
		     for(j=0;j<Num_states_;j++)
		     {
			    EstadoG.SAL2.Q_table[i][j] = 0;
		     }
	      }
	      load_dialog(2);
       }
       gtk_widget_destroy(dialog_warn);
}

/**
* @fn void on_SAL1_bt_save_Q_clicked(GtkWidget *widget, gpointer user_data)
* 
*/
void on_SAL1_bt_save_Q_clicked(GtkWidget *widget, gpointer user_data)
{
       save_dialog(1);
}

/**
* @fn void on_SAL2_bt_save_Q_clicked(GtkWidget *widget, gpointer user_data)
* 
*/
void on_SAL2_bt_save_Q_clicked(GtkWidget *widget, gpointer user_data)
{
       save_dialog(2);
}

/**
* @fn on_SAL1_bt_load_learn_pgr_clicked(GtkWidget * widget, gpointer user_data)
* CallBack que trata a resposta ao clique deste botão
* Carrega os dados relativos à progressão de Aprendizagem do SAL1 de um ficheiro
*/
void on_SAL1_bt_load_learn_pgr_clicked(GtkWidget *widget, gpointer user_data)
{   
       int i;
       GtkWidget *parent_win = gtk_window_new(GTK_WINDOW_POPUP);
       GtkWidget *dialog_warn = gtk_message_dialog_new((GtkWindow *)parent_win,
													GTK_DIALOG_MODAL,
													GTK_MESSAGE_WARNING,
													GTK_BUTTONS_YES_NO,
													"Deseja realmente carregar valores da progressão da Aprendizagem?\nOs valores actuais do SALbot 1 irão ser substituidos.");
       
       if(gtk_dialog_run(GTK_DIALOG(dialog_warn))==GTK_RESPONSE_YES)
       {
	      EstadoG.SAL1.Learn_iteration = 0;
	      for(i=0;i<65535;i++)
	      {
		     EstadoG.SAL1.Learn_progress[i] = 0;
	      }
	      gfloat min_x, max_x, min_y, max_y;
	      min_x =  -11;
	      min_y =  -11;
	      max_x = 111;
	      max_y = 111;
	      
	      DataBoxG.SAL1_max_x = 100;
	      DataBoxG.SAL1_max_y = 100;
	      
	      gtk_databox_set_total_limits(GTK_DATABOX(DataBoxG.SAL1_box),min_x,max_x,max_y,min_y);
	      load_pgr_dialog(1);
	      Actualiza_graph_SAL1();
       }
       gtk_widget_destroy(dialog_warn);
}

/**
* @fn on_SAL2_bt_load_learn_pgr_clicked(GtkWidget * widget, gpointer user_data)
* CallBack que trata a resposta ao clique deste botão
* Carrega os dados relativos à progressão de Aprendizagem do SAL2 de um ficheiro
*/
void on_SAL2_bt_load_learn_pgr_clicked(GtkWidget *widget, gpointer user_data)
{   
       int i;
       GtkWidget *parent_win = gtk_window_new(GTK_WINDOW_POPUP);
       GtkWidget *dialog_warn = gtk_message_dialog_new((GtkWindow *)parent_win,
													GTK_DIALOG_MODAL,
													GTK_MESSAGE_WARNING,
													GTK_BUTTONS_YES_NO,
													"Deseja realmente carregar valores da progressão da Aprendizagem?\nOs valores actuais do SALbot 2 irão ser substituidos.");
       
       if(gtk_dialog_run(GTK_DIALOG(dialog_warn))==GTK_RESPONSE_YES)
       {
	      EstadoG.SAL2.Learn_iteration = 0;
	      for(i=0;i<65535;i++)
	      {
		     EstadoG.SAL2.Learn_progress[i] = 0;
	      }
	      gfloat min_x, max_x, min_y, max_y;
	      min_x =  -11;
	      min_y =  -11;
	      max_x = 111;
	      max_y = 111;
	      
	      DataBoxG.SAL2_max_x = 100;
	      DataBoxG.SAL2_max_y = 100;
	      
	      gtk_databox_set_total_limits(GTK_DATABOX(DataBoxG.SAL2_box),min_x,max_x,max_y,min_y);
	      load_pgr_dialog(2);
	      Actualiza_graph_SAL2();
       }
       gtk_widget_destroy(dialog_warn);
}

/**
* @fn on_SAL1_bt_save_learn_pgr_clicked(GtkWidget * widget, gpointer user_data)
* CallBack que trata a resposta ao clique deste botão
* Guarda os dados relativos à progressão de Aprendizagem do SAL1 num ficheiro
*/
void on_SAL1_bt_save_learn_pgr_clicked(GtkWidget *widget, gpointer user_data)
{
       save_pgr_dialog(1);
}

/**
* @fn on_SAL1_bt_save_learn_pgr_clicked(GtkWidget * widget, gpointer user_data)
* CallBack que trata a resposta ao clique deste botão
* Guarda os dados relativos à progressão de Aprendizagem do SAL1 num ficheiro
*/
void on_SAL2_bt_save_learn_pgr_clicked(GtkWidget *widget, gpointer user_data)
{
       save_pgr_dialog(2);
}

/**
* @fn void on_SAL1_bt_learn_begin_clicked(GtkWidget *widget, gpointer user_data)
* 
*/
void on_SAL1_bt_learn_begin_clicked(GtkWidget *widget, gpointer user_data)
{
       int pid = getppid();
       
       GtkWidget *parent_win = gtk_window_new(GTK_WINDOW_POPUP);
       GtkWidget *dialog_warn = gtk_message_dialog_new((GtkWindow *)parent_win,
							GTK_DIALOG_MODAL,
							GTK_MESSAGE_WARNING,
							GTK_BUTTONS_YES_NO,
							"Deseja realmente iniciar uma nova aprendizagem no SAL1?\nOs dados relativos à aprendizagem anterior irão ser apagados!");
							
       if(gtk_dialog_run(GTK_DIALOG(dialog_warn))==GTK_RESPONSE_YES)
       {
	      while(!pf_msgG->request_to_send){}
	      pf_msgG->send_msg.SALbot_dest = SALbot1;
	      pf_msgG->send_msg.Tipo = op_mode_cmd;
	      pf_msgG->send_msg.Cmd = learn_begin;
	      
	      pf_msgG->request_to_send = FALSE;
	      kill(pid,SIGUSR1);
       }
       gtk_widget_destroy(dialog_warn);
}

/**
* @fn void on_SAL2_bt_learn_begin_clicked(GtkWidget *widget, gpointer user_data)
* 
*/
void on_SAL2_bt_learn_begin_clicked(GtkWidget *widget, gpointer user_data)
{
       int pid = getppid();
       
       GtkWidget *parent_win = gtk_window_new(GTK_WINDOW_POPUP);
       GtkWidget *dialog_warn = gtk_message_dialog_new((GtkWindow *)parent_win,
							GTK_DIALOG_MODAL,
							GTK_MESSAGE_WARNING,
							GTK_BUTTONS_YES_NO,
							"Deseja realmente iniciar uma nova aprendizagem no SAL2?\nOs dados relativos à aprendizagem anterior irão ser apagados!");
							
       if(gtk_dialog_run(GTK_DIALOG(dialog_warn))==GTK_RESPONSE_YES)
       {
	      while(!pf_msgG->request_to_send){}
	      pf_msgG->send_msg.SALbot_dest = SALbot2;
	      pf_msgG->send_msg.Tipo = op_mode_cmd;
	      pf_msgG->send_msg.Cmd = learn_begin;
	      
	      pf_msgG->request_to_send = FALSE;
	      kill(pid,SIGUSR1);
       }
       gtk_widget_destroy(dialog_warn);
}

/**
* @fn void on_bt_sair_clicked(GtkWidget *widget, gpointer user_data)
 * Função chamada sempre que o utilizador clique no botão de Sair.\n
 * Guarda na \c shared \c memory o valor Exit, para que o Pai saiba que é
 * para terminar o programa, escreve na consola que vai sair e termina
 * a interface gráfica.
 * @param widget Corresponde ao botão pressionado.
 * @param user_data Ponteiro para dados do botão.
 */
void on_bt_sair_clicked(GtkWidget *widget, gpointer user_data)
{
	pf_msgG->Com = Exit;
	g_print("Vou Sair!!!\n");
	gtk_main_quit();
}


/**
* @fn on_imagemenuitem5_activate (GtkWidget *widget, gpointer user_data)
* Identico ao do botão Sair.
* @param widget Corresponde ao botão pressionado.
* @param user_data Ponteiro para dados do botão.
*/
void on_imagemenuitem5_activate (GtkWidget *widget, gpointer user_data)
{
       pf_msgG->Com = Exit;
       g_print("Vou Sair!!!\n");
       gtk_main_quit();
}

/**
* Nova Função.........................................................................................:::::::::::::::::::::::::::::::::::
*/

void expose_event_callback(GtkWidget *wdg, GdkEventExpose *event, gpointer data)
{
       //draw_point(wdg, 100, 100, 1, 1, 250);
       //draw_point(wdg, 200, 200, 250, 1, 1);
}
