/**
* @file Child.c
* @author Hugo Tavares nº 27764 (hugo.tavares@ua.pt)
* @brief Ficheiro que contém o processo Filho
* 
* Este ficheiro contém o processo Filho que trata da interface gráfica e da mecânica
* do jogo.
*/

#define _CHILD_

#include "SALbot_Interface.h"

/*!
* @fn int childMain(str_msg* pf_mem, int argc , char *argv[])
* Função Main do Filho.
* Carrega a interface gráfica e cria as tabelas de jogo.
* @param pf_mem \c Shared \c memory.
* @param argc inteiro que representa o número de argumentos agregados ao comando para executar o programa.
* @param argv[] porteiro para \c arrays de caracteres (ponteiro para strings).
* Contem todos os argumentos do comando que o utilizador introduziu para iniciar o programa.
*/
int childMain(str_msg* pf_mem, int argc , char *argv[])
{
       /**
       * Informa o processo que quando receber sinais em \c SIGUSR1
       * tem de correr a rotina \c childSignalHandler (interrupt)
       */
	signal(SIGUSR1, childSignalHandler);
	signal(SIGINT, childKillHandler);
	
	int pid = getppid();
	
	printline();
		
	pf_msgG->Com = Receber;
	pf_msgG->request_to_send = TRUE;
	EstadoG.SAL1.Modo_op = Idle;
	EstadoG.SAL2.Modo_op = Idle;
	EstadoG.SAL1.Learn_iteration = 0;
	EstadoG.SAL2.Learn_iteration = 0;
	EstadoG.SAL1.Learn_progress[0] = 0;
	EstadoG.SAL2.Learn_progress[0] = 0;
	EstadoG.SAL1.Learn_epsilon[0] = 0.5;
	EstadoG.SAL2.Learn_epsilon[0] = 0.5;
	
	GladeXML *xml;
	GtkWidget *Vbox;
	gtk_init(&argc,&argv);
	
	/**
	  * Cria a interface gráfica.
	  */
	xml =  glade_xml_new("SALbot_interface.glade",NULL,NULL);
	xmlG = xml;
	
	
	/**
	 * Criar gráficos de aprendizagem
	 */	
	gtk_databox_create_box_with_scrollbars_and_rulers(&DataBoxG.SAL1_box,&DataBoxG.SAL1_boxcontainer,TRUE,TRUE,TRUE,TRUE);
	Vbox = glade_xml_get_widget(xmlG,"Vbox_SAL1");
	gtk_box_pack_start(GTK_BOX(Vbox),DataBoxG.SAL1_boxcontainer,1,1,0);
	gtk_databox_create_box_with_scrollbars_and_rulers(&DataBoxG.SAL2_box,&DataBoxG.SAL2_boxcontainer,TRUE,TRUE,TRUE,TRUE);
	Vbox = glade_xml_get_widget(xmlG,"Vbox_SAL2");
	gtk_box_pack_start(GTK_BOX(Vbox),DataBoxG.SAL2_boxcontainer,1,1,0);
	
	GdkColor cor;
	GtkDataboxGraph *graph;
	gfloat min_x, max_x, min_y, max_y;
	
	
	EstadoG.SAL1.X = g_new0(gfloat,65535);
	EstadoG.SAL1.Y = g_new0(gfloat,65535);
	EstadoG.SAL2.X = g_new0(gfloat,65535);
	EstadoG.SAL2.Y = g_new0(gfloat,65535);
	
	/* Cor de fundo dos gráficos */
	
	cor.red = 65535;
	cor.green = 65535;
	cor.blue = 65535;
	
	gtk_widget_modify_bg(DataBoxG.SAL1_box, GTK_STATE_NORMAL, &cor);
	gtk_widget_modify_bg(DataBoxG.SAL2_box, GTK_STATE_NORMAL, &cor);
	
	cor.red = 0;
	cor.green = 0;
	cor.blue = 0;
	
	graph = gtk_databox_cross_simple_new(&cor,0);
	gtk_databox_graph_add(GTK_DATABOX(DataBoxG.SAL1_box),graph);
	//g_signal_connect(GTK_OBJECT(DataBoxG.SAL1_box),"key_press_event",G_CALLBACK(SAL1_key_pressed),graph);
	gtk_databox_graph_add(GTK_DATABOX(DataBoxG.SAL2_box),graph);
	//g_signal_connect(GTK_OBJECT(DataBoxG.SAL2_box),"key_press_event",G_CALLBACK(SAL2_key_pressed),graph);
	
	min_x =  -11;
	min_y =  -11;
	max_x = 111;
	max_y = 111;
	
	DataBoxG.SAL1_max_x = 100;
	DataBoxG.SAL1_max_y = 100;
	DataBoxG.SAL2_max_x = 100;
	DataBoxG.SAL2_max_y = 100;
	
	DataBoxG.SAL1_view.maxv_x = 111;
	DataBoxG.SAL1_view.minv_x = -11;
	DataBoxG.SAL1_view.maxv_y = 111;
	DataBoxG.SAL1_view.minv_y = -11;
	
	DataBoxG.SAL2_view.maxv_x = 111;
	DataBoxG.SAL2_view.minv_x = -11;
	DataBoxG.SAL2_view.maxv_y = 111;
	DataBoxG.SAL2_view.minv_y = -11;
	
	gtk_databox_set_total_limits(GTK_DATABOX(DataBoxG.SAL1_box),min_x,max_x,max_y,min_y);
	gtk_databox_set_total_limits(GTK_DATABOX(DataBoxG.SAL2_box),min_x,max_x,max_y,min_y);
	
	gtk_widget_add_events (DataBoxG.SAL1_box, GDK_KEY_PRESS_MASK);
	GTK_WIDGET_SET_FLAGS (DataBoxG.SAL1_box, GTK_CAN_FOCUS | GTK_CAN_DEFAULT); 
	gtk_widget_add_events (DataBoxG.SAL2_box, GDK_KEY_PRESS_MASK);
	GTK_WIDGET_SET_FLAGS (DataBoxG.SAL2_box, GTK_CAN_FOCUS | GTK_CAN_DEFAULT); 
	gtk_widget_grab_default(DataBoxG.SAL1_box);
	gtk_widget_grab_default(DataBoxG.SAL2_box);
	
	
	/**
	* Carrega as imagens necessárias para a interface do Jogo.
	*/
	GError **error=NULL;
	SALbot_bumper_pixbufG.Front_left.open = gdk_pixbuf_new_from_file("SALbot_interface img/Front_left.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Front_left.closed = gdk_pixbuf_new_from_file("SALbot_interface img/Front_left_x.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Front_right.open = gdk_pixbuf_new_from_file("SALbot_interface img/Front_right.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Front_right.closed = gdk_pixbuf_new_from_file("SALbot_interface img/Front_right_x.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Left_front.open = gdk_pixbuf_new_from_file("SALbot_interface img/Left_front.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Left_front.closed = gdk_pixbuf_new_from_file("SALbot_interface img/Left_front_x.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Right_front.open = gdk_pixbuf_new_from_file("SALbot_interface img/Right_front.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Right_front.closed = gdk_pixbuf_new_from_file("SALbot_interface img/Right_front_x.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Left.open = gdk_pixbuf_new_from_file("SALbot_interface img/Left.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Left.closed = gdk_pixbuf_new_from_file("SALbot_interface img/Left_x.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Right.open = gdk_pixbuf_new_from_file("SALbot_interface img/Right.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Right.closed = gdk_pixbuf_new_from_file("SALbot_interface img/Right_x.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Back_left.open = gdk_pixbuf_new_from_file("SALbot_interface img/Back_left.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Back_left.closed = gdk_pixbuf_new_from_file("SALbot_interface img/Back_left_x.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Back_right.open = gdk_pixbuf_new_from_file("SALbot_interface img/Back_right.png",error);
	error=NULL;
	SALbot_bumper_pixbufG.Back_right.closed = gdk_pixbuf_new_from_file("SALbot_interface img/Back_right_x.png",error);
	error=NULL;
	
	
	SALbot_mov_pixbufG.no_mov = gdk_pixbuf_new_from_file("SALbot_interface img/No_mov.png",error);
	error=NULL;
	SALbot_mov_pixbufG.mov_front = gdk_pixbuf_new_from_file("SALbot_interface img/Front.png",error);
	error=NULL;
	SALbot_mov_pixbufG.mov_back = gdk_pixbuf_new_from_file("SALbot_interface img/Back.png",error);
	error=NULL;
	SALbot_mov_pixbufG.turn_left = gdk_pixbuf_new_from_file("SALbot_interface img/Roll_left.png",error);
	error=NULL;
	SALbot_mov_pixbufG.turn_right = gdk_pixbuf_new_from_file("SALbot_interface img/Roll_right.png",error);
	error=NULL;
	SALbot_mov_pixbufG.mov_left_f = gdk_pixbuf_new_from_file("SALbot_interface img/Turn_left_front.png",error);
	error=NULL;
	SALbot_mov_pixbufG.mov_right_f = gdk_pixbuf_new_from_file("SALbot_interface img/Turn_right_front.png",error);
	error=NULL;
	SALbot_mov_pixbufG.mov_left_b = gdk_pixbuf_new_from_file("SALbot_interface img/Turn_left_back.png",error);
	error=NULL;
	SALbot_mov_pixbufG.mov_right_b = gdk_pixbuf_new_from_file("SALbot_interface img/Turn_right_back.png",error);
	error=NULL;
	
	/**
	* Conecta os \c signals com a interface.
	*/
	glade_xml_signal_autoconnect(xml);
	
	/**
	* Inicia a interface gráfica em loop, até o utilizador fechar a janela.
	*/
	gtk_main();

	pf_msgG->send_msg.Tipo = start_stop_cmd;
	pf_msgG->send_msg.Cmd = stop_cmd;
	kill(pid,SIGUSR1);
	
	while(pf_msgG->Com == Enviar)
	{
	}
	
	while(pf_msgG->Com == Receber)
	{
	}
	
	
	g_print("Mandei o Pai Sair!!!\n");
	
	/**
	  * Saída do programa(PAI)
	  */
	pf_msgG->Com = Exit;
	
	/**
	* "Desatacha" a \c shared \c memory da estrutura de dados pf_mem.
	*/
	shmdt((void*)pf_mem);
	
	return 0;
}

/**
* @fn void childSignalHandler(int dumm)
* Interrupt activado pelo \c SIGUSR1.
* Permite ao Filho avaliar o resultado dos comandos enviados pelo Pai.
*/
void childSignalHandler(int dumm)
{
       int pid = getppid();
       
       /**
       * Escreve na consola do terminal o comando que o Pai recebeu.
       */
	textcolor(RESET,GREEN,-30);
	printf("Filho: ");
	textcolor(RESET,WHITE,GREEN);
	
	/**
	* Verifica que tipo de mensagem recebeu
	*/
	switch(pf_msgG->Msg.tipo)
	{
	       case 1: //Sensores distância
		      switch(pf_msgG->Robot)
		      {
			     case 1: //SALbot1
				    EstadoG.SAL1.IR_F.val = ((unsigned char) pf_msgG->Sens_dist.IR_F_valH * 0x100) + ((unsigned char) pf_msgG->Sens_dist.IR_F_valL);
				    EstadoG.SAL1.IR_B.val = ((unsigned char) pf_msgG->Sens_dist.IR_B_valH * 0x100) + ((unsigned char) pf_msgG->Sens_dist.IR_B_valL);
				    EstadoG.SAL1.IR_L.val = ((unsigned char) pf_msgG->Sens_dist.IR_L_valH * 0x100) + ((unsigned char) pf_msgG->Sens_dist.IR_L_valL);
				    EstadoG.SAL1.IR_R.val = ((unsigned char) pf_msgG->Sens_dist.IR_R_valH * 0x100) + ((unsigned char) pf_msgG->Sens_dist.IR_R_valL);
				    EstadoG.SAL1.US_F.val = ((unsigned char) pf_msgG->Sens_dist.US_F_valH * 0x100) + ((unsigned char) pf_msgG->Sens_dist.US_F_valL);
				    
				    /*  Conversão do valor lido dos sensores para distância em mm */
				    EstadoG.SAL1.IR_F.dist = gp2d120_count_to_mm_a*(pow((double)EstadoG.SAL1.IR_F.val,(float)(gp2d120_count_to_mm_b)));
				    EstadoG.SAL1.IR_B.dist = gp2d120_count_to_mm_a*(pow((double)EstadoG.SAL1.IR_B.val,(float)(gp2d120_count_to_mm_b)));
				    EstadoG.SAL1.IR_L.dist = gp2d120_count_to_mm_a*(pow((double)EstadoG.SAL1.IR_L.val,(float)(gp2d120_count_to_mm_b)));
				    EstadoG.SAL1.IR_R.dist = gp2d120_count_to_mm_a*(pow((double)EstadoG.SAL1.IR_R.val,(float)(gp2d120_count_to_mm_b)));
				    EstadoG.SAL1.US_F.dist = lv_ez0_count_to_mm*EstadoG.SAL1.US_F.val;
				    
				    if(EstadoG.SAL1.US_F.val >= US_F_Level1_)
				    {
					   EstadoG.SAL1.US_F.level = 2;
				    }
				    else if(EstadoG.SAL1.US_F.val >= US_F_Level2_)
				    {
					   EstadoG.SAL1.US_F.level = 1;
				    }
				    else
				    {
					   EstadoG.SAL1.US_F.level = 0;
				    }
				    
				    if(EstadoG.SAL1.IR_F.val >= IR_F_Level3_)
				    {
					   EstadoG.SAL1.IR_F.level = 3;
				    }
				    else if(EstadoG.SAL1.IR_F.val >= IR_F_Level2_)
				    {
					   EstadoG.SAL1.IR_F.level = 2;
				    }
				    else if(EstadoG.SAL1.IR_F.val >= IR_F_Level1_)
				    {
					   EstadoG.SAL1.IR_F.level = 1;
				    }
				    else
				    {
					   EstadoG.SAL1.IR_F.level = 0;
				    }
				    
				    if(EstadoG.SAL1.IR_B.val >= IR_B_Level2_)
				    {
					   EstadoG.SAL1.IR_B.level = 2;
				    }
				    else if(EstadoG.SAL1.IR_B.val >= IR_B_Level1_)
				    {
					   EstadoG.SAL1.IR_B.level = 1;
				    }
				    else
				    {
					   EstadoG.SAL1.IR_B.level = 0;
				    }
				    
				    if(EstadoG.SAL1.IR_L.val >= IR_L_Level3_)
				    {
					   EstadoG.SAL1.IR_L.level = 3;
				    }
				    else if(EstadoG.SAL1.IR_L.val >= IR_L_Level2_)
				    {
					   EstadoG.SAL1.IR_L.level = 2;
				    }
				    else if(EstadoG.SAL1.IR_L.val >= IR_L_Level1_)
				    {
					   EstadoG.SAL1.IR_L.level = 1;
				    }
				    else
				    {
					   EstadoG.SAL1.IR_L.level = 0;
				    }
				    
				    if(EstadoG.SAL1.IR_R.val >= IR_R_Level3_)
				    {
					   EstadoG.SAL1.IR_R.level = 3;
				    }
				    else if(EstadoG.SAL1.IR_R.val >= IR_R_Level2_)
				    {
					   EstadoG.SAL1.IR_R.level = 2;
				    }
				    else if(EstadoG.SAL1.IR_R.val >= IR_R_Level1_)
				    {
					   EstadoG.SAL1.IR_R.level = 1;
				    }
				    else
				    {
					   EstadoG.SAL1.IR_R.level = 0;
				    }
				    Actualiza_lbl_SAL1();
				    break;
			     case 2: //SALbot2
				    EstadoG.SAL2.IR_F.val = ((unsigned char) pf_msgG->Sens_dist.IR_F_valH * 0x100) + ((unsigned char) pf_msgG->Sens_dist.IR_F_valL);
				    EstadoG.SAL2.IR_B.val = ((unsigned char) pf_msgG->Sens_dist.IR_B_valH * 0x100) + ((unsigned char) pf_msgG->Sens_dist.IR_B_valL);
				    EstadoG.SAL2.IR_L.val = ((unsigned char) pf_msgG->Sens_dist.IR_L_valH * 0x100) + ((unsigned char) pf_msgG->Sens_dist.IR_L_valL);
				    EstadoG.SAL2.IR_R.val = ((unsigned char) pf_msgG->Sens_dist.IR_R_valH * 0x100) + ((unsigned char) pf_msgG->Sens_dist.IR_R_valL);
				    EstadoG.SAL2.US_F.val = ((unsigned char) pf_msgG->Sens_dist.US_F_valH * 0x100) + ((unsigned char) pf_msgG->Sens_dist.US_F_valL);
				    
				    /*  Conversão do valor lido dos sensores para distância em mm */
				    EstadoG.SAL2.IR_F.dist = gp2d120_count_to_mm_a*(pow((double)EstadoG.SAL2.IR_F.val,(float)(gp2d120_count_to_mm_b)));
				    EstadoG.SAL2.IR_B.dist = gp2d120_count_to_mm_a*(pow((double)EstadoG.SAL2.IR_B.val,(float)(gp2d120_count_to_mm_b)));
				    EstadoG.SAL2.IR_L.dist = gp2d120_count_to_mm_a*(pow((double)EstadoG.SAL2.IR_L.val,(float)(gp2d120_count_to_mm_b)));
				    EstadoG.SAL2.IR_R.dist = gp2d120_count_to_mm_a*(pow((double)EstadoG.SAL2.IR_R.val,(float)(gp2d120_count_to_mm_b)));
				    EstadoG.SAL2.US_F.dist = lv_ez0_count_to_mm*EstadoG.SAL2.US_F.val;
				    
				    
				    if(EstadoG.SAL2.US_F.val >= US_F_Level1_)
				    {
					   EstadoG.SAL2.US_F.level = 2;
				    }
				    else if(EstadoG.SAL2.US_F.val >= US_F_Level2_)
				    {
					   EstadoG.SAL2.US_F.level = 1;
				    }
				    else
				    {
					   EstadoG.SAL2.US_F.level = 0;
				    }
				    
				    if(EstadoG.SAL2.IR_F.val >= IR_F_Level3_)
				    {
					   EstadoG.SAL2.IR_F.level = 3;
				    }
				    else if(EstadoG.SAL2.IR_F.val >= IR_F_Level2_)
				    {
					   EstadoG.SAL2.IR_F.level = 2;
				    }
				    else if(EstadoG.SAL2.IR_F.val >= IR_F_Level1_)
				    {
					   EstadoG.SAL2.IR_F.level = 1;
				    }
				    else
				    {
					   EstadoG.SAL2.IR_F.level = 0;
				    }
				    
				    if(EstadoG.SAL2.IR_B.val >= IR_B_Level2_)
				    {
					   EstadoG.SAL2.IR_B.level = 2;
				    }
				    else if(EstadoG.SAL2.IR_B.val >= IR_B_Level1_)
				    {
					   EstadoG.SAL2.IR_B.level = 1;
				    }
				    else
				    {
					   EstadoG.SAL2.IR_B.level = 0;
				    }
				    
				    if(EstadoG.SAL2.IR_L.val >= IR_L_Level3_)
				    {
					   EstadoG.SAL2.IR_L.level = 3;
				    }
				    else if(EstadoG.SAL2.IR_L.val >= IR_L_Level2_)
				    {
					   EstadoG.SAL2.IR_L.level = 2;
				    }
				    else if(EstadoG.SAL2.IR_L.val >= IR_L_Level1_)
				    {
					   EstadoG.SAL2.IR_L.level = 1;
				    }
				    else
				    {
					   EstadoG.SAL2.IR_L.level = 0;
				    }
				    
				    if(EstadoG.SAL2.IR_R.val >= IR_R_Level3_)
				    {
					   EstadoG.SAL2.IR_R.level = 3;
				    }
				    else if(EstadoG.SAL2.IR_R.val >= IR_R_Level2_)
				    {
					   EstadoG.SAL2.IR_R.level = 2;
				    }
				    else if(EstadoG.SAL2.IR_R.val >= IR_R_Level1_)
				    {
					   EstadoG.SAL2.IR_R.level = 1;
				    }
				    else
				    {
					   EstadoG.SAL2.IR_R.level = 0;
				    }
				    Actualiza_lbl_SAL2();
				    break;
		      }
		      break;
	       case 2:  //Sensores para-choques
		      switch(pf_msgG->Robot)
		      {
			     case 1: //SALbot1
				    EstadoG.SAL1.Bumper.back_right =	(unsigned char) pf_msgG->Sens_bumper & 0x1;
				    EstadoG.SAL1.Bumper.back_left =	((unsigned char) pf_msgG->Sens_bumper ) >> 1 & 0x1;
				    EstadoG.SAL1.Bumper.front_right =	((unsigned char) pf_msgG->Sens_bumper ) >> 2 & 0x1;
				    EstadoG.SAL1.Bumper.front_left =	((unsigned char) pf_msgG->Sens_bumper ) >> 3 & 0x1;
				    EstadoG.SAL1.Bumper.right_front =	((unsigned char) pf_msgG->Sens_bumper ) >> 4 & 0x1;
				    EstadoG.SAL1.Bumper.left_front =	((unsigned char) pf_msgG->Sens_bumper ) >> 5 & 0x1;
				    EstadoG.SAL1.Bumper.right =		((unsigned char) pf_msgG->Sens_bumper ) >> 6 & 0x1;
				    EstadoG.SAL1.Bumper.left =			((unsigned char) pf_msgG->Sens_bumper ) >> 7 & 0x1;
				    Actualiza_img_SAL1();
				    break;
			     case 2: //SALbot2
				    EstadoG.SAL2.Bumper.back_right =	(unsigned char) pf_msgG->Sens_bumper & 0x1;
				    EstadoG.SAL2.Bumper.back_left =	((unsigned char) pf_msgG->Sens_bumper ) >> 1 & 0x1;
				    EstadoG.SAL2.Bumper.front_right =	((unsigned char) pf_msgG->Sens_bumper ) >> 2 & 0x1;
				    EstadoG.SAL2.Bumper.front_left =	((unsigned char) pf_msgG->Sens_bumper ) >> 3 & 0x1;
				    EstadoG.SAL2.Bumper.right_front =	((unsigned char) pf_msgG->Sens_bumper ) >> 4 & 0x1;
				    EstadoG.SAL2.Bumper.left_front =	((unsigned char) pf_msgG->Sens_bumper ) >> 5 & 0x1;
				    EstadoG.SAL2.Bumper.right =		((unsigned char) pf_msgG->Sens_bumper ) >> 6 & 0x1;
				    EstadoG.SAL2.Bumper.left =			((unsigned char) pf_msgG->Sens_bumper ) >> 7 & 0x1;
				    Actualiza_img_SAL2();
				    break;
		      }
		      break;
		      
	       case 3: //Tabela Aprendizagem
	       {
		      unsigned int action_, state_, Addr_;
		      
		      Addr_ = ((unsigned char)pf_msgG->Q_Table.Addr_H * 0x100) + ((unsigned char)pf_msgG->Q_Table.Addr_L);
		      action_ = Addr_ % Num_actions_;
		      state_ = Addr_ / Num_actions_;
		      
		      switch(pf_msgG->Robot)
		      {
			     case 1: //SALbot1
				    EstadoG.SAL1.Q_table[action_][state_] = (signed char)pf_msgG->Q_Table.val;
				    break;
			     case 2: //SALbot2
				    EstadoG.SAL2.Q_table[action_][state_] = (signed char)pf_msgG->Q_Table.val;
				    break;
		      }
		      break;
	       }
	       case 4: //Progresso de Aprendizagem
	       {
		      unsigned int Iteration, Colisions;
		      unsigned int eps;
		      double Epsilon;
		      
		      Iteration = ((unsigned char)pf_msgG->Learn_pgr.iteration_H * 0x100) + ((unsigned char)pf_msgG->Learn_pgr.iteration_L);
		      Colisions = ((unsigned char)pf_msgG->Learn_pgr.n_colisions_H * 0x100) + ((unsigned char)pf_msgG->Learn_pgr.n_colisions_L);
		      eps = ((unsigned char)pf_msgG->Learn_pgr.epsilon_H * 0x100) + ((unsigned char)pf_msgG->Learn_pgr.epsilon_L); 
		      Epsilon = (double)(eps/1000.0);
		      switch(pf_msgG->Robot)
		      {
			     case 1:
				    EstadoG.SAL1.Learn_iteration = Iteration;
				    EstadoG.SAL1.Learn_progress[Iteration] = Colisions;
				    EstadoG.SAL1.Learn_epsilon[Iteration] = Epsilon;
				    printf("\nEpsilon = %1.3f\n",Epsilon);
				    Actualiza_graph_SAL1();
				    break;
			     case 2:
				    EstadoG.SAL2.Learn_iteration = Iteration;
				    EstadoG.SAL2.Learn_progress[Iteration] = Colisions;
				    EstadoG.SAL2.Learn_epsilon[Iteration] = Epsilon;
				    printf("\nEpsilon = %1.3f\n",Epsilon);
				    Actualiza_graph_SAL2();
				    break;
		      }
		      break;
	       }
	       case 5: //Acção executada
		      switch(pf_msgG->Robot)
		      {
			     case 1:
				    EstadoG.SAL1.Action = pf_msgG->Action_e;
				    Actualiza_img_action_SAL1();
				    break;
			     case 2:
				    EstadoG.SAL2.Action = pf_msgG->Action_e;
				    Actualiza_img_action_SAL2();
				    break;
		      }
		      break;
	       
	       case 6: // Estado_SAL
		      switch(pf_msgG->Robot)
		      {
			     case 1:
				    EstadoG.SAL1.Modo_op = pf_msgG->Estado_;
				    Actualiza_estado_SAL1();
				    break;
			     case 2:
				    EstadoG.SAL2.Modo_op = pf_msgG->Estado_;
				    Actualiza_estado_SAL2();
				    break;
		      }
		      break;
				    
	       default:
		      break;
	}
	
	resetcolor();
	printf("\n");
	printline();
}

/**
* @fn gboolean on_window1_map_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
* Interrupt gerado quando a janela da interface gráfica arranca(ou quando está minimizada e é reposta).
* @param widget Widget que sofreu o evento.
* @param event tipo de evento ocorrido.
* @param user_data ponteiro para dados associados ao evento.
*/
gboolean on_window1_map_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
       GtkWidget *note_book = glade_xml_get_widget(xmlG,"notebook1"); 
       
       gtk_widget_show_all(GTK_WIDGET(note_book));
       
       Actualiza_estado_SAL1();
       Actualiza_estado_SAL2();
}

/**
* @fn gboolean on_window2_map_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
* Interrupt gerado quando a janla 2 da interface gráfica arranca(ou quando está minimizada e é reposta).
* @param widget Widget que sofreu o evento.
* @param event tipo de evento ocorrido.
* @param user_data ponteiro para dados associados ao evento.
*/
gboolean on_window2_map_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
       GtkWidget *img = glade_xml_get_widget(xmlG,"img_PARI");
       GdkPixbuf *img_PARI;
       GError **error=NULL;
       
        img_PARI = gdk_pixbuf_new_from_file("Pics/PARI_small.png",error);
	gtk_image_set_from_pixbuf(GTK_IMAGE(img), img_PARI);
       
}

/**
* @fn void childKillHandler(int dumm)
* Interrupt activado pelo \c SIGINT (ctrl +c).
* @brief
* Permite interceptar o sinal de interrupção do programa, evitando assim que o Filho
* termine a sua execução.
*/
void childKillHandler(int dumm)
{
       textcolor(RESET,GREEN,-30);
       printf("Filho: ");
       textcolor(RESET,WHITE,GREEN);
       printf("Não quero morrer já!!");
       resetcolor();
       printf("\n");
       printline();
}
