/**
 * @mainpage Projecto em Automação e Robótica Industrial - Rato Óptico
 *
 *
 * @section intro_sec Introdução
 *
 * O objecivo deste trabalho é desenvolver uma aplicação em \c GTK  e \c C que consiste numa interface que mostra
 * o caminho percorrido pelo rato óptico. O rato envia mensagens pela porta série RS232, que o Pai recebe.
 * Ao receber uma mensagem pela porta série o Pai envia ao Filho a mensagem e este, ao recebê-la, vai
 * interpreta-la e actualizar a interface perante a mensagem que recebeu.
 * \n \n \n \n
 * @section prom_sec Descrição
 *
 * Este programa comunica com um dispositivo que faz a leitura de um Rato e de dois sensores de distância.\n
 * Nessa comunicação, através de um protocolo específico, o dispositivo envia dados que correspondem aos valores
 * lidos dos dispositivos (sensores e rato) que este programa vai interpretar e mostra-los num ambiente gráfico
 * para que o utilizador possa enterder melhor o estado desses dispositivos.\n
 * O utilizador pode ainda controlar o modo de operação do dispositivo primário, guardar os dados actuais, carregar dados
 * antigos ou ainda limpar os dados actuais.
 *  \n
 * @section autor_sec Autor
 *
 * Hugo Emanuel Fonseca Tavares nº mec.: 27764
 *
 * e-mail: hugo.tavares@ua.pt
 * \n \n \n
 */
 
/**
 * @file Main.c
 * @author Hugo Tavares nº 27764 (hugo.tavares@ua.pt)
 * @brief Rotina principal do MouseSens
 *
 * 
 */

#define _MAIN_

#include "SALbot_Interface.h"

int main(int argc, char *argv[])
{
       int pc;
       
       if(argc != 2)
       {
	      header();
	      textcolor(RESET,BLUE,-30);
	      printf("\nInrtoduza o comando MouseSens + ( ");
	      textcolor(RESET,WHITE,BLUE);
	      printf("[U]");
	      textcolor(RESET,RED,-30);
	      printf(" ou ");
	      textcolor(RESET,WHITE,YELLOW);
	      printf("[W]");
	      textcolor(RESET,RED,-30);
	      printf(" ou ");
	      textcolor(RESET,WHITE,GREEN);
	      printf("[S]");
	      textcolor(RESET,BLUE,-30);
	      printf(" )\n\n");
	      resetcolor();
	      textcolor(RESET,WHITE,BLUE);
	      printf("[U]");
	      textcolor(RESET,CYAN,-30);
	      printf(" - Para comunicação pela porta ttyUSB0\n\n");
	      textcolor(RESET,WHITE,YELLOW);
	      printf("[W]");
	      textcolor(RESET,CYAN,-30);
	      printf(" - Para comunicação pela porta ttyUSB0\n\n");
	      textcolor(RESET,WHITE,GREEN);
	      printf("[S]");
	      textcolor(RESET,CYAN,-30);
	      printf(" - Para comunicação pela porta ttyS0\n\n");
	      resetcolor();
	      return 0;
       }
       else if (strcmp(argv[1],"S")==0 || strcmp(argv[1],"s")==0)
	      pc = 1;
       else if (strcmp(argv[1],"U")==0 || strcmp(argv[1],"u")==0)
	      pc = 2;
       else if (strcmp(argv[1],"W")==0 || strcmp(argv[1],"w")==0)
	      pc = 3;
       else
	      return 0;
       
       key_t key = ftok("Main.c",255);/**< É uma chave única que foi obtida através da função \c ftok */
       if(key == -1)
       {
	      perror("ftok");
	      exit(1);
       }
       
       /**
       * @code
       * int SHM_id = shmget(key,SHM_size,IPC_CREAT | 0644);
       * É o Id da \c shared \c memory
       * @endcode
       */
	int SHM_id = shmget(key,SHM_size,IPC_CREAT | 0644);
	if(SHM_id < 0)
	{
	       perror("shmget");
	       exit(1);
	}
	
	/*!
	* Estrutura de dados partilhada
	*/
	str_msg *pf_mem = (str_msg*)shmat(SHM_id,(void*)0,0);
	if(pf_mem == (str_msg*)NULL)
	{
	       perror("shmat");
	       exit(1);
	}
	
	/**
	* Estrutura de dados partilhada Global
	*/
	pf_msgG = pf_mem;
	
	pf_msgG->Com = Receber;
	
	/**
	* Biforca o programa em dois processos o Pai e o Filho.
	* O Pai fica encarregue de fazer as comunicações pela porta RS232,
	* Enquanto que o Filho fica encarregue da inteface do jogo, bem como
	* da mecânica do jogo.
	*/
	int pid = fork();
	
	switch (pid)
	{
	       case (-1):
		      printf("Erro: Não foi possivel biforcar!\nSaindo..\n");
		      return -1;
	       case 0:
		      textcolor(RESET,GREEN,-30);
		      printf("Eu sou o ");
		      textcolor(RESET,WHITE,GREEN);
		      printf("Filho!");
		      resetcolor();
		      printf("\n");
		      childMain(pf_mem, argc , argv);
		      break;
	       default:
		      textcolor(RESET,BLUE,-30);
		      printf("Eu sou o ");
		      textcolor(RESET,WHITE,BLUE);
		      printf("Pai!");
		      resetcolor();
		      printf("\n");
		      parentMain(pid,pf_mem,pc);
		      break;
	}
	
	/**
	* "Desatacha" a \c shared \c memory da estrutura de dados pf_mem
	*/
	shmdt((void*)pf_mem);
	
	/**
	* Liberta a \c shared \c memory
	*/
	shmctl(SHM_id,IPC_RMID,(struct shmid_ds*)NULL);
	pf_msgG = 0;
}
