/**
 * @file Parent_Fun.c
 * @author Hugo Tavares nº 27764 (hugo.tavares@ua.pt)
 * @brief Ficheiro que contém funções auxiliares ao processo Pai
 * 
 * Este ficheiro contém a função que identifica o tipo de comando 
 * recebido pelo Pai.
 */


#define _PARENT_FUN_

#include "SALbot_Interface.h"

/**
* @fn sfc(int pid,unsigned char* str)
 * Função que identifica o tipo de comando enviado pela comunicação série.
 * Depois de identificar o comando recebido, vai colocar o que recebeu na 
 * \c shared \c memory para o Filho ter acesso ao comando e o possa interpertar
 * e envia-lhe um sinal para lhe informar que recebeu um comando.
 * @param pid pid do Filho
 * @param str mensagem recebida (string)
 */
int sfc(int pid,unsigned char* str)
{
	switch(pf_msgG->Msg.tipo)
	{	       
	       case Sensors_dist:
		      textcolor(RESET,BLUE,-30);
		      printf("Pai: ");
		      textcolor(RESET,WHITE,BLUE);
		      printf("Foram recebidos dados dos sensores de distância.");
		      resetcolor();
		      printf("\n");
		      pf_msgG->Sens_dist.IR_F_valH =	(unsigned char)str[2] ;
		      pf_msgG->Sens_dist.IR_F_valL = 	(unsigned char)str[3] ;
		      pf_msgG->Sens_dist.IR_B_valH = 	(unsigned char)str[4] ;
		      pf_msgG->Sens_dist.IR_B_valL = 	(unsigned char)str[5] ;
		      pf_msgG->Sens_dist.IR_L_valH = 	(unsigned char)str[6] ;
		      pf_msgG->Sens_dist.IR_L_valL = 	(unsigned char)str[7] ;
		      pf_msgG->Sens_dist.IR_R_valH = 	(unsigned char)str[8] ;
		      pf_msgG->Sens_dist.IR_R_valL = 	(unsigned char)str[9] ;
		      pf_msgG->Sens_dist.US_F_valH = 	(unsigned char)str[10] ;
		      pf_msgG->Sens_dist.US_F_valL = 	(unsigned char)str[11] ;
		      kill(pid,SIGUSR1);
		      break;
	       case Sensors_bumper:
		      textcolor(RESET,BLUE,-30);
		      printf("Pai: ");
		      textcolor(RESET,WHITE,BLUE);
		      printf("Foram recebidos dados do estado do sistema de para-choques.");
		      resetcolor();
		      printf("\n");
		      pf_msgG->Sens_bumper = 	(unsigned char)str[2];
		      kill(pid,SIGUSR1);
		      break;
	       case Q_table:
		      textcolor(RESET,BLUE,-30);
		      printf("Pai: ");
		      textcolor(RESET,WHITE,BLUE);
		      printf("Foram recebidos dados da tabela de Aprendizagem.");
		      resetcolor();
		      printf("\n");
		      pf_msgG->Q_Table.Addr_H =	(unsigned char)str[2];
		      pf_msgG->Q_Table.Addr_L =	(unsigned char)str[3];
		      pf_msgG->Q_Table.val =		(unsigned char)str[4];
		      kill(pid,SIGUSR1);
		      break;
	       case  Learn_progress:
		      textcolor(RESET,BLUE,-30);
		      printf("Pai: ");
		      textcolor(RESET,WHITE,BLUE);
		      printf("Foram recebidos dados do progresso de Aprendizagem.");
		      resetcolor();
		      printf("\n");
		      pf_msgG->Learn_pgr.iteration_H =		(unsigned char)str[2];
		      pf_msgG->Learn_pgr.iteration_L =		(unsigned char)str[3];
		      pf_msgG->Learn_pgr.n_colisions_H =	(unsigned char)str[4];
		      pf_msgG->Learn_pgr.n_colisions_L =	(unsigned char)str[5];
		      pf_msgG->Learn_pgr.epsilon_H =		(unsigned char)str[6];
		      pf_msgG->Learn_pgr.epsilon_L =		(unsigned char)str[7];
		      kill(pid,SIGUSR1);
		      break;
	       case Action_executed:
		      textcolor(RESET,BLUE,-30);
		      printf("Pai: ");
		      textcolor(RESET,WHITE,BLUE);
		      printf("Foi recebido a acção executada de um dos robôs.");
		      resetcolor();
		      printf("\n");
		      pf_msgG->Action_e =	(unsigned char)str[2];
		      kill(pid,SIGUSR1);
		      break;
	       case Estado_SAL:
		      textcolor(RESET,BLUE,-30);
		      printf("Pai: ");
		      textcolor(RESET,WHITE,BLUE);
		      printf("Foi recebido o estado de um dos robôs.");
		      resetcolor();
		      printf("\n");
		      pf_msgG->Estado_ =	(unsigned char)str[2];
		      kill(pid,SIGUSR1);
		      break;
	       default:
		      break;
	}
}


/**
* @fn Processa_msg(void)
* Função que processa a mensagem a enviar. \n
* Isto é, cria a mensagem a enviar pedida pelo Filho no formato protocolar.
*/
unsigned char Processa_msg(void)
{
       int type;
       unsigned char Tipo, byte_check, cmd;
       unsigned char it_H, it_L, col_H, col_L, eps_H, eps_L;
       
       type = pf_msgG->send_msg.Tipo;
       cmd = pf_msgG->send_msg.Cmd;
       
       switch(pf_msgG->send_msg.SALbot_dest)
       {
	      case 1:
		     pf_msgG->send_msg.msg[0] = 255;
		     break;
	      case 2:
		     pf_msgG->send_msg.msg[0] = 254;
		     break;
       }
       
       switch(type)
       {
	      case 4:
		     Tipo = op_mode_cmd_len;
		     Tipo = Tipo << 4;
		     Tipo += type;
		     byte_check = Tipo;
		     byte_check ^= cmd;
		     
		     pf_msgG->send_msg.msg[1] = Tipo;
		     pf_msgG->send_msg.msg[2] = cmd;
		     pf_msgG->send_msg.msg[3] = byte_check;
		     pf_msgG->send_msg.len = 4;
		     break;
	      
	      case 5:

		     Tipo = req_cmd_len;
		     Tipo = Tipo << 4;
		     Tipo += type;
		     byte_check = Tipo;
		     byte_check ^= cmd;
		     
		     pf_msgG->send_msg.msg[1] = Tipo;
		     pf_msgG->send_msg.msg[2] = cmd;
		     pf_msgG->send_msg.msg[3] = byte_check;
		     pf_msgG->send_msg.len = 4;
		     break;
		     
	      case 7:
		     
		     Tipo = q_table_cmd_len;
		     Tipo = Tipo << 4;
		     Tipo += type;
		     byte_check = Tipo;
		     byte_check ^= pf_msgG->send_msg.Q_send.Addr_H;
		     byte_check ^= pf_msgG->send_msg.Q_send.Addr_L;
		     byte_check ^= pf_msgG->send_msg.Q_send.val;
		     
		     pf_msgG->send_msg.msg[1] = Tipo;
		     pf_msgG->send_msg.msg[2] = pf_msgG->send_msg.Q_send.Addr_H;
		     pf_msgG->send_msg.msg[3] = pf_msgG->send_msg.Q_send.Addr_L;
		     pf_msgG->send_msg.msg[4] = pf_msgG->send_msg.Q_send.val;
		     pf_msgG->send_msg.msg[5] = byte_check;
		     pf_msgG->send_msg.len = 6;
		     break;
		     
	      case 8:
		     
		     it_H = (pf_msgG->send_msg.Pgr_send.It >> 8) & 0xFF;
		     it_L = pf_msgG->send_msg.Pgr_send.It & 0xFF;
		     col_H = (pf_msgG->send_msg.Pgr_send.Col >> 8) & 0xFF;
		     col_L = pf_msgG->send_msg.Pgr_send.Col & 0xFF;
		     eps_H = (pf_msgG->send_msg.Pgr_send.Eps >> 8) & 0xFF;
		     eps_L = pf_msgG->send_msg.Pgr_send.Eps & 0xFF;
		     
		     Tipo = learn_pgr_len;
		     Tipo = Tipo << 4;
		     Tipo += type;
		     byte_check = Tipo;
		     byte_check ^= it_H;
		     byte_check ^= it_L;
		     byte_check ^= col_H;
		     byte_check ^= col_L;
		     byte_check ^= eps_H;
		     byte_check ^= eps_L;
		     
		     pf_msgG->send_msg.msg[1] = Tipo;
		     pf_msgG->send_msg.msg[2] = it_H;
		     pf_msgG->send_msg.msg[3] = it_L;
		     pf_msgG->send_msg.msg[4] = col_H;
		     pf_msgG->send_msg.msg[5] = col_L;
		     pf_msgG->send_msg.msg[6] = eps_H;
		     pf_msgG->send_msg.msg[7] = eps_L;
		     pf_msgG->send_msg.msg[8] = byte_check;
		     pf_msgG->send_msg.len = 9;
		     break;
		     
	      case 15:
		     
		     Tipo = start_stop_cmd_len;
		     Tipo = Tipo << 4;
		     Tipo += type;
		     byte_check = Tipo;
		     byte_check ^= cmd;
		     
		     pf_msgG->send_msg.msg[1] = Tipo;
		     pf_msgG->send_msg.msg[2] = cmd;
		     pf_msgG->send_msg.msg[3] = byte_check;
		     pf_msgG->send_msg.len = 4;
		     break;
       }
       
}
