/**
  *@file SALbot_Interface.h
  *@author Hugo Tavares nº 27764 (hugo.tavares@ua.pt)
  *@brief Cabeçalho do programa MouseSens
  *
  *Neste cabeçalho estão incluidos os "includes", "defines", "macros", 
  *estruturas e protótipos do programa MouseSens
  **/

#ifndef _SALbot_INTER_
#define _SALbot_INTER_
#endif

//__________________________________________________
//::::::::::::::::::::::::::::::::::::::::::::: Includes ::::::::::::::::::::::::::::::::::::::::::::::::::
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/**
* @name Includes dos vários ficheiros sources
*/
/*@{*/
#if defined _CHILD_ || defined _SALbot_INTER_ || defined _AUX_ || defined _CALL_BACK_
#include <gtk/gtk.h>
#include <glade/glade.h>
// NEW------------------------------------------------
#include <gtkdatabox.h>
#include <gtkdatabox_points.h>
#include <gtkdatabox_lines.h>
#include <gtkdatabox_bars.h>
#include <gtkdatabox_cross_simple.h>
#include <gtkdatabox_ruler.h>
// --------------------------------------------------------
#endif

#if defined _CHILD_ || defined _PARENT_ || defined _CALL_BACK_
#include <sys/shm.h>
#include <signal.h>
#include <unistd.h>
#include <math.h>
#include <gdk/gdkkeysyms.h>
#endif

#if defined _PRINT_ || defined _TEXT_
#include <stdio.h>
#endif

#ifdef _MAIN_
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#endif
 
#ifdef _AUX_
#include <sys/stat.h>
#include <sys/signal.h>
#endif
 
#ifdef _PARENT_FUN_
#include <signal.h>
#endif

#ifdef _FUN_
#include <sys/time.h>
#endif

#ifdef _PARENT_
#include <fcntl.h>
#include <termios.h>
#endif

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <stdbool.h>
/*@}*/

//__________________________________________________
//:::::::::::::::::::::::::::::::::::::::::::::: Defines :::::::::::::::::::::::::::::::::::::::::::::::::::
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

/**
* @name Definição dos atributos da função textcolor
*/
/*@{*/
#define RESET 0
#define BRIGHT 1
#define DIM 2
#define UNDERLINE 3
#define BLINK 4
#define REVERSE 7
#define HIDDEN 8
/*@}*/

/**
* @name Definição das cores da função textcolor
*/
/*@{*/
#define BLACK 0
#define RED 1
#define GREEN 2
#define YELLOW 3
#define BLUE 4
#define MAGENTA 5
#define CYAN 6
#define WHITE 7
/*@}*/

/**
* Desinição do tamanho da \c shared \c memory
*/
#define SHM_size sizeof(str_msg)

/**
 * Definição das variáveis para a conversão de unidades de contagem do Rato para mm.\n \n
 * Do \e Datasheet da \b AVAGO do sensor <a href="../../../Docs/ADNS-2610.pdf" target="_new">\b ADNS-2610 </a> retiraram-se os seguintes dados:\n
 * \b Resolução: \c 400 \c counts \c per \c in \n
 * \b Cadência \b de \b movimento: \c 12 \c in/s \n
 * Sabendo que \f$ 1\mbox{in} = 25,4 \mbox{mm} \f$ e
 * \f[
 * 	\frac{\mbox{counts}}{\mbox{in}} \times \frac{\mbox{in}}{\mbox{mm}} = \frac{\mbox{counts}}{\mbox{mm}};
 * \f]
 * ,então:
 * \f[
 * 	400 \times \frac{1}{25,4} = 15,74801496;
 * \f]
 * \f[
 * 	\frac{\mbox{mm}}{\mbox{counts}} = 0,063500067;
 * \f]
 */
#define	mouse_count_to_mm	(float)0.063500067
/**
* Definição das variáveis para a conversão de unidades de contagem do \c LV-EZ0 para mm.\n \n
* Do site da <a href="http://www.maxbotix.com/MaxSonar-EZ1__FAQ.html" target="_new">\b Maxbotix </a> retirou-se a esacala de medida: \f$ \frac{\mbox{Vcc}}{512}  \frac{\mbox{V}}{\mbox{in}} \f$.
* Sabendo que \f$ \mbox{V}_{\mbox{cc}} = 5\mbox{V} \f$ , \f$ 5\mbox{V} = 1024 \mbox{ counts} \f$, \f$ 1\mbox{in} = 25,4 \mbox{mm} \f$ e que
* \f[
* 	\frac{\frac{\mbox{V}}{\mbox{counts}}}{\frac{\mbox{V}}{\mbox{in}} \times \frac{\mbox{in}}{\mbox{mm}}} = \frac{\mbox{mm}}{\mbox{counts}};
* \f]
* ,então:
* \f[
* 	 \frac{\mbox{mm}}{\mbox{counts}} = \frac{\frac{5}{1024}}{\frac{5}{512} \times \frac{1}{25,4}};
* \f]
* \f[
* 	 \frac{\mbox{mm}}{\mbox{counts}} = 12,7;
* \f]
* 
*/
#define	lv_ez0_count_to_mm	(float)12.7
/**
* @name Definição das variáveis para a conversão de unidades de contagem do GP2D120 para mm.
*/
/*@{*/
/**
 * \image html GP2D120_graph.png
 * A partir dos dados retirados do \e Datasheet da \b SHARP do sensor <a href="../../../Docs/gp2d120.pdf" target="_new"> \b GP2D120 </a> realizou-se este gráfico do qual se retirou
 * a  seguinte fórmula matemática:
 * \f[
 * 	\mbox{y} = \mbox{a} \times \mbox{x}^{-\mbox{b}}   \mbox{[mm]};
 * \f]
 * 
 * \f[
 * 	\mbox{y} = 37866,277869 \times \mbox{x}^{-1,073405} \mbox{[mm]};
 * \f]
 *  Que corresponde, apróximadamente, à distância \e (em \e mm \e )
 * em função do valor do valor convertido de analógico para digital da saída do sensor.\n
 * Na qual
 * @c gp2d120_count_to_mm_a corresponde à variável \c a e
 * @c gp2d120_count_to_mm_b correponde à variável \c b .
 * 
 */
#define	gp2d120_count_to_mm_a	(float)37866.277869
#define	gp2d120_count_to_mm_b	(float)-1.073405
/*@}*/

#define round(num)	(num > 0.0) ? floor(num + 0.5) : ceil(num - 0.5)

/**
* @name Definição dos comprimentos das mensagens a enviar
 */
/*@{*/
#define op_mode_cmd_len	1
#define req_cmd_len			1
#define start_stop_cmd_len	1
#define q_table_cmd_len		3
#define learn_pgr_len			6
/*@}*/

/**
 *@name Definição dos níveis de distância dos sensores
 */
/*@{*/
// De 12 a 1023 (155mm a 6450mmm)
#define		US_F_Level1_	47
#define		US_F_Level2_	39
// De 775 a 69 (30mm a 400mm)
#define		IR_F_Level1_	140
#define		IR_F_Level2_	377
#define		IR_F_Level3_	395
// De 775 a 69 (30mm a 400mm)
#define		IR_B_Level1_	390
#define		IR_B_Level2_	400
// De 775 a 69 (30mm a 400mm)
#define		IR_L_Level1_	100
#define		IR_L_Level2_	305	
#define		IR_L_Level3_	476	
// De 775 a 69 (30mm a 400mm)
#define		IR_R_Level1_	150	
#define		IR_R_Level2_	290
#define		IR_R_Level3_	465
/*@}*/

#define		Num_actions_	6
#define		Num_states_		9216
//#define		Num_addr_		Num_actions_ * Num_states_

//__________________________________________________
//:::::::::::::::::::::::::::::::::::::::::::::::: Enum ::::::::::::::::::::::::::::::::::::::::::::::::::::
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

/**
* @enum Comunic
* Enumeração do tipo de comunicação.
*/
typedef enum
{
       Enviar,  /*!< Envia*/
       Receber, /*!< Recebe*/
       Exit     /*!< Fecha a comunicação*/
} Comunic;

typedef enum
{
       Autonomo=1,
       Manual,
       Learn_begin,
       Learning,
       Idle
} modo_op_enum;

typedef enum
{
       none,
       Sensors_dist,
       Sensors_bumper,
       Q_table,
       Learn_progress,
       Action_executed,
       Estado_SAL
} tipo_msg;

typedef enum
{
       op_mode_cmd=4,
       req_cmd,
       send_Q=7,
       send_pgr,
       start_stop_cmd=15
} type_send_enum;

typedef enum
{
       autonomo=1,
       manual,
       learn_begin,
       learning,
       learn_read,
       sens_dist,
       sens_bumper,
       Q_reset,
       start_cmd=85,
       stop_cmd=170
} cmd_send_enum;

typedef enum
{
       SALbot1=1,
       SALbot2
} robot_enum;

//__________________________________________________
//:::::::::::::::::::::::::::::::::::::::::::: Estruturas ::::::::::::::::::::::::::::::::::::::::::::::::
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

typedef struct
{
       unsigned char	Addr_H;
       unsigned char	Addr_L;
       unsigned char	val;
} Q_msg_struct;

typedef struct
{
       unsigned int		It;
       unsigned int		Col;
       unsigned int		Eps;
} Learn_pgr_struct;

typedef struct
{
       robot_enum		SALbot_dest;
       type_send_enum	Tipo;
       cmd_send_enum	Cmd;
       Q_msg_struct		Q_send;
       Learn_pgr_struct	Pgr_send;
       unsigned int		len;
       unsigned char	msg[20];
} send_struct;

typedef struct
{
       GdkPixbuf	*open;
       GdkPixbuf	*closed;
} bumper_state;

typedef struct
{
       bumper_state		Front_left;
       bumper_state		Front_right;
       bumper_state		Left_front;
       bumper_state		Right_front;
       bumper_state		Left;
       bumper_state		Right;
       bumper_state		Back_left;
       bumper_state		Back_right;
} Estado_bumper;

typedef struct
{
       GdkPixbuf	*no_mov;
       GdkPixbuf	*mov_front;
       GdkPixbuf	*mov_back;
       GdkPixbuf	*turn_left;
       GdkPixbuf	*turn_right;
       GdkPixbuf	*mov_left_f;
       GdkPixbuf	*mov_right_f;
       GdkPixbuf	*mov_right_b;
       GdkPixbuf	*mov_left_b;
} Pixbuf_mov_str;

typedef struct
{
       bool		front_left;
       bool		front_right;
       bool		left_front;
       bool		right_front;
       bool		left;
       bool		right;
       bool		back_left;
       bool		back_right;
} bumper_str;

typedef struct
{
       unsigned char		iteration_H;
       unsigned char		iteration_L;
       unsigned char		n_colisions_H;
       unsigned char		n_colisions_L;
       unsigned char		epsilon_H;
       unsigned char		epsilon_L;       
} progress_struct;

typedef struct
{
       unsigned char	IR_F_valH;
       unsigned char	IR_F_valL;
       unsigned char	IR_B_valH;
       unsigned char	IR_B_valL;
       unsigned char	IR_L_valH;
       unsigned char	IR_L_valL;
       unsigned char	IR_R_valH;
       unsigned char	IR_R_valL;
       unsigned char	US_F_valH;
       unsigned char	US_F_valL;
} Dist_msg_struct;

typedef struct
{
       tipo_msg		tipo;
       //int			len;
       //int			byte_test;
} msg_struct;

typedef struct
{
       gfloat minv_x, maxv_x, minv_y, maxv_y ;
} view_str;

typedef struct
{
       unsigned int		val;
       float				dist;
       unsigned int		level;
} Dist_struct;

typedef struct
{
       GtkWidget 		*SAL1_boxcontainer;
       GtkWidget		*SAL1_box;
       GtkWidget		*SAL2_boxcontainer;
       GtkWidget		*SAL2_box;
       gfloat			SAL1_max_x;
       gfloat			SAL1_max_y;
       gfloat			SAL2_max_x;
       gfloat			SAL2_max_y;
       view_str			SAL1_view;
       view_str			SAL2_view;
} Graph_struct;

typedef struct
{
       Dist_struct		IR_F, IR_B, IR_L, IR_R, US_F;
       bumper_str		Bumper;
       signed char		Q_table[Num_actions_][Num_states_];
       unsigned int		Learn_progress[0xFFFF];
       unsigned int		Learn_iteration;
       float				Learn_epsilon[0xFFFF];
       gfloat			*X,*Y;
       unsigned char	Action;
       modo_op_enum	Modo_op; 
} SALbot_struct;

/**
* @struct str_msg
* Estrutura de dados para a \c shared \c memory.\n
* Esta estrutura de dados contém informações sobre qual o tipo de comunicação
* a realizar (receber ou enviar), o que foi recebido e o que foi lido. 
*/
typedef struct
{
       bool					request_to_send;
       msg_struct			Msg;
       send_struct			send_msg;
       Dist_msg_struct		Sens_dist;
       unsigned char		Sens_bumper;
       progress_struct		Learn_pgr;
       unsigned char		Action_e;
       Q_msg_struct			Q_Table;
       modo_op_enum		Estado_;	
       robot_enum			Robot;
       Comunic 			Com;     /*!< Tipo de comunicação*/
} str_msg;

typedef struct
{
       SALbot_struct	SAL1;
       SALbot_struct	SAL2;
} str_estado;

//__________________________________________________
//::::::::::::::::::::::::::::::::::::::::::::::: Extern :::::::::::::::::::::::::::::::::::::::::::::::::::
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#ifdef _MAIN_
#define EXTERN 
#else
#define EXTERN extern
#endif

EXTERN str_msg *pf_msgG;

#undef EXTERN

#ifdef _CHILD_
#define EXTERN 
#else
#define EXTERN extern
#endif

EXTERN GladeXML  *xmlG;
EXTERN str_estado EstadoG;
EXTERN Estado_bumper SALbot_bumper_pixbufG;
EXTERN Pixbuf_mov_str SALbot_mov_pixbufG;
EXTERN Graph_struct DataBoxG;

#undef EXTERN

//__________________________________________________
//:::::::::::::::::::::::::::::::::::::::::::: Protótipos ::::::::::::::::::::::::::::::::::::::::::::::::
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include "proto.h"
