/**
* @file UART.h
* @author Hugo Tavares nº 27764 (hugo.tavares@ua.pt)
* @brief Cabeçalho das funções que permitem o envio e recepção de dados pela porta série.
*
* 
*/

/*********************************************************************
 *
 *                  UART Routines
 *
 *********************************************************************
 * FileName:        UART.h
 * Dependencies:
 * Processor:       PIC18
 * Complier:        HI-TECH PICC18
 ********************************************************************/
#ifndef  _UART_H_
#define _UART_H_

#include "generic.h"
#include <htc.h>

#if defined(__PICC18__)
#if 1   // Useful for disabling the console (saving power)
    void UARTInit(void);
    #define UARTIsPutReady()     (TRMT)
    void UARTPut(BYTE);
    void UARTPutString(BYTE *);
    void UARTPut_String(unsigned char*);

    #define UARTIsGetReady()     (RCIF)
    BYTE UARTGet(void);
    BYTE UARTGetString(char *buffer, BYTE bufferLen);
#else
    #define UARTInit()
    #define UARTIsPutReady() 1
    #define UARTPut(c)
    #define UARTPutString(s)
    #define UARTPut_String(str)

    #define UARTIsGetReady() 1
    #define UARTGet()        'a'
    #define UARTGetString(buffer, bufferLen) 0
#endif

#else
	#error Unknown processor.  See htc.h
#endif

//TODO: remove test code
//<test code>
void PrintChar(BYTE);
//</test code>
#define printf(x)	UARTPut_String((unsigned char *)x)

#endif

